@@ -125,6 +125,7 @@ public:
 class MainDialog : public Lum::Dialog
 {
 private:
+  GameModelRef                  gameModel;
   Lum::Model::ActionRef         clearAction;
   Lum::Model::ActionRef         generateAction;
   Lum::Model::ActionRef         markEmptyAction;
@@ -136,6 +137,7 @@ private:
   GenerateTask                  generateTask;
   Lum::Model::BooleanRef        keypad[Sudoku::dimension];
   Lum::Model::BooleanRef        mark;
+  bool                          ignoreFinished;
   bool                          lock;
   Lum::Images::StatefulImageRef hintImage;
   Lum::Images::StatefulImageRef statisticsImage;
@@ -144,21 +146,27 @@ private:
 
 public:
   MainDialog()
-  : clearAction(new Lum::Model::Action),
+  : gameModel(new GameModel()),
+    clearAction(new Lum::Model::Action),
     generateAction(new Lum::Model::Action),
     markEmptyAction(new Lum::Model::Action),
     solveAction(new Lum::Model::Action),
     statAction(new Lum::Model::Action),
     helpAction(new Lum::Model::Action),
     aboutAction(new Lum::Model::Action),
-    gameArea(new GameArea()),
+    gameArea(NULL),
     mark(new Lum::Model::Boolean(false)),
+    ignoreFinished(false),
     lock(false),
     hintImage(new Lum::Images::StatefulImage()),
     statisticsImage(new Lum::Images::StatefulImage()),
     solveImage(new Lum::Images::StatefulImage()),
     generateImage(new Lum::Images::StatefulImage())
   {
+    GetWindow()->SetScreenOrientationHint(Lum::OS::Window::screenOrientationBothSupported);
+
+    Observe(gameModel);
+
     for (size_t i=0; i<Sudoku::dimension; i++) {
       keypad[i]=new Lum::Model::Boolean(false);
       Observe(keypad[i]);
@@ -180,9 +188,6 @@ public:
     Observe(GetClosedAction());
     Observe(generateTask.GetAbortedAction());
     Observe(generateTask.GetFinishedAction());
-    gameArea->GetFinishedModel()->Off();
-    Observe(gameArea->GetFocusChangedAction());
-    Observe(gameArea->GetFinishedModel());
 
     Lum::Images::loader->SetNormalImagePostfix(L"");
     Lum::Images::loader->LoadApplication(L"Hint.png",hintImage);
@@ -193,92 +198,193 @@ public:
 
   void PreInit()
   {
-    Lum::Array *array;
-    Lum::Panel *horiz,*vert;
-
-    horiz=Lum::HPanel::Create(true,true);
-
+    gameArea=new GameArea();
     gameArea->SetFlex(true,true);
-    gameArea->SetModel(riddle,solution);
+    gameArea->SetModel(gameModel);
+    //gameArea->SetModel(riddle,solution);
 
-    horiz->Add(gameArea);
+    if (Lum::OS::display->IsLandscape()) {
+      Lum::Array *array;
+      Lum::Panel *horiz,*vert;
 
-    horiz->AddSpace();
+      horiz=Lum::HPanel::Create(true,true);
+      horiz->Add(gameArea);
 
-    vert=Lum::VPanel::Create(false,true);
+      horiz->AddSpace();
+
+      vert=Lum::VPanel::Create(false,true);
+
+      if (true/*Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()*/) {
+        array=new Lum::Array();
+        array->SetFlex(true,true);
+        array->SetHorizontalCount(3);
+        array->SetSpace(false,false);
+
+        for (size_t i=0; i<Sudoku::dimension; i++) {
+          Lum::Text *text=new Lum::Text(Lum::Base::NumberToWString(i+1),
+                                        Lum::OS::Font::normal,
+                                        Lum::Text::left,
+                                        Lum::OS::display->GetFont(Lum::OS::Display::fontScaleLogo));
+
+          array->Add(Lum::Toggle::Create(text,keypad[i],true,true));
+        }
+
+        vert->Add(array);
+
+        array=new Lum::Array();
+        array->SetFlex(true,false);
+        array->SetHorizontalCount(2);
+        array->SetSpace(false,false);
+
+        array->Add(Lum::Button::Create(_(L"MAIN_CLEAR",L"Clear"),clearAction,true,true));
+        array->Add(Lum::Toggle::Create(_(L"MAIN_MARK",L"Mark"),mark,true,true));
+
+        vert->Add(array);
+
+        vert->AddSpace();
+      }
+      else {
+        vert->AddSpace(true);
+      }
 
-    if (true/*Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()*/) {
       array=new Lum::Array();
-      array->SetFlex(true,true);
-      array->SetHorizontalCount(3);
+      array->SetFlex(true,false);
+      array->SetHorizontalCount(4);
       array->SetSpace(false,false);
 
-      for (size_t i=0; i<Sudoku::dimension; i++) {
-        array->Add(Lum::Toggle::Create(Lum::Base::NumberToWString(i+1),keypad[i],true,true));
+      if (hintImage->IsValid()) {
+        Lum::Toggle* toggle=new Lum::Toggle();
+
+        toggle->SetFlex(true,false);
+        toggle->SetModel(gameArea->GetHintMode());
+        toggle->SetLabel(Lum::Image::Create(hintImage));
+
+        array->Add(toggle);
+      }
+      else {
+        array->Add(Lum::Toggle::Create(_(L"GAME_HINT",L"Hint"),gameArea->GetHintMode(),true,false));
+      }
+
+      if (statisticsImage->IsValid()) {
+        array->Add(Lum::Button::Create(Lum::Image::Create(statisticsImage),statAction,true,false));
+      }
+      else {
+        array->Add(Lum::Button::Create(_(L"GAME_STAT",L"S_tatistics"),statAction,true,false));
+      }
+
+      if (solveImage->IsValid()) {
+        array->Add(Lum::Button::Create(Lum::Image::Create(solveImage),solveAction,true,false));
+      }
+      else {
+        array->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,true,false));
+      }
+
+      if (generateImage->IsValid()) {
+        array->Add(Lum::Button::Create(Lum::Image::Create(generateImage),generateAction,true,false));
+      }
+      else {
+        array->Add(Lum::Button::Create(_(L"GENERATE",L"_New"),generateAction,true,false));
       }
 
       vert->Add(array);
 
-      array=new Lum::Array();
-      array->SetFlex(true,false);
-      array->SetHorizontalCount(2);
-      array->SetSpace(false,false);
+      horiz->Add(vert);
 
-      array->Add(Lum::Button::Create(_(L"MAIN_CLEAR",L"Clear"),clearAction,true,true));
-      array->Add(Lum::Toggle::Create(_(L"MAIN_MARK",L"Mark"),mark,true,true));
+      SetMain(horiz);
+    }
+    else {
+      Lum::Array *array;
+      Lum::Panel *vert;
+      Lum::Panel *horiz;
+      Lum::Panel *vert2;
 
-      vert->Add(array);
+      vert=Lum::VPanel::Create(true,true);
+
+      vert->Add(gameArea);
 
       vert->AddSpace();
-    }
-    else {
-      vert->AddSpace(true);
-    }
 
-    array=new Lum::Array();
-    array->SetFlex(true,false);
-    array->SetHorizontalCount(4);
-    array->SetSpace(false,false);
+      horiz=Lum::HPanel::Create(true,false);
 
-    if (hintImage->IsValid()) {
-      Lum::Toggle* toggle=new Lum::Toggle();
+      vert2=Lum::VPanel::Create(false,true);
 
-      toggle->SetFlex(true,false);
-      toggle->SetModel(gameArea->GetHintMode());
-      toggle->SetLabel(Lum::Image::Create(hintImage));
+      if (hintImage->IsValid()) {
+        Lum::Toggle* toggle=new Lum::Toggle();
 
-      array->Add(toggle);
+        toggle->SetFlex(false,true);
+        toggle->SetModel(gameArea->GetHintMode());
+        toggle->SetLabel(Lum::Image::Create(hintImage));
+
+        vert2->Add(toggle);
+      }
+      else {
+        vert2->Add(Lum::Toggle::Create(_(L"GAME_HINT",L"Hint"),gameArea->GetHintMode(),false,true));
+      }
+
+      if (statisticsImage->IsValid()) {
+        vert2->Add(Lum::Button::Create(Lum::Image::Create(statisticsImage),statAction,false,true));
+      }
+      else {
+        vert2->Add(Lum::Button::Create(_(L"GAME_STAT",L"S_tatistics"),statAction,false,true));
+      }
+
+      if (solveImage->IsValid()) {
+        vert2->Add(Lum::Button::Create(Lum::Image::Create(solveImage),solveAction,false,true));
+      }
+      else {
+        vert2->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,false,true));
+      }
+
+      if (generateImage->IsValid()) {
+        vert2->Add(Lum::Button::Create(Lum::Image::Create(generateImage),generateAction,false,true));
+      }
+      else {
+        vert2->Add(Lum::Button::Create(_(L"GENERATE",L"_New"),generateAction,false,true));
+      }
+
+      horiz->Add(vert2);
+
+      horiz->AddSpace();
+
+      vert2=Lum::VPanel::Create(true,true);
+
+      if (true/*Lum::OS::display->GetTheme()->RequestFingerFriendlyControls()*/) {
+        array=new Lum::Array();
+        array->SetFlex(true,true);
+        array->SetHorizontalCount(3);
+        array->SetSpace(false,false);
+
+        for (size_t i=0; i<Sudoku::dimension; i++) {
+          Lum::Text *text=new Lum::Text(Lum::Base::NumberToWString(i+1),
+                                        Lum::OS::Font::normal,
+                                        Lum::Text::left,
+                                        Lum::OS::display->GetFont(Lum::OS::Display::fontScaleLogo));
+
+          array->Add(Lum::Toggle::Create(text,keypad[i],true,true));
+        }
+
+        vert2->Add(array);
+
+        array=new Lum::Array();
+        array->SetFlex(true,false);
+        array->SetHorizontalCount(2);
+        array->SetSpace(false,false);
+
+        array->Add(Lum::Button::Create(_(L"MAIN_CLEAR",L"Clear"),clearAction,true,true));
+        array->Add(Lum::Toggle::Create(_(L"MAIN_MARK",L"Mark"),mark,true,true));
+
+        vert2->Add(array);
+      }
+      else {
+        vert2->AddSpace(true);
+      }
+
+      horiz->Add(vert2);
+
+      vert->Add(horiz);
+
+      SetMain(vert);
     }
-    else {
-      array->Add(Lum::Toggle::Create(_(L"GAME_HINT",L"Hint"),gameArea->GetHintMode(),true,false));
-    }
-
-    if (statisticsImage->IsValid()) {
-      array->Add(Lum::Button::Create(Lum::Image::Create(statisticsImage),statAction,true,false));
-    }
-    else {
-      array->Add(Lum::Button::Create(_(L"GAME_STAT",L"S_tatistics"),statAction,true,false));
-    }
-
-    if (solveImage->IsValid()) {
-      array->Add(Lum::Button::Create(Lum::Image::Create(solveImage),solveAction,true,false));
-    }
-    else {
-      array->Add(Lum::Button::Create(_(L"SOLVE",L"_Solve"),solveAction,true,false));
-    }
-
-    if (generateImage->IsValid()) {
-      array->Add(Lum::Button::Create(Lum::Image::Create(generateImage),generateAction,true,false));
-    }
-    else {
-      array->Add(Lum::Button::Create(_(L"GENERATE",L"_New"),generateAction,true,false));
-    }
-
-    vert->Add(array);
-
-    horiz->Add(vert);
-
-    SetMain(horiz);
 
     Lum::Def::Menu *menu=Lum::Def::Menu::Create();
 
@@ -315,18 +421,17 @@ public:
 
   void GenerateRiddle()
   {
-    riddle.Clear();
-    gameArea->GetFinishedModel()->Off();
-    gameArea->SetModel(riddle,solution);
+    gameModel->Clear();
 
     generateTask.Start();
   }
 
   void UpdateKeypad()
   {
-    if (gameArea==NULL ||
-        gameArea->GetFinishedModel()->Get() ||
-        !gameArea->IsFocusedCellEditable()) {
+    if (gameModel==NULL ||
+        !gameModel->IsValid() ||
+        gameModel->IsFinished() ||
+        !gameModel->IsFocusedCellEditable()) {
       for (size_t i=0; i<Sudoku::dimension; i++) {
         if (keypad[i].Valid()) {
           keypad[i]->Set(false);
@@ -346,8 +451,8 @@ public:
       return;
     }
 
-    if (gameArea->IsCellInMarkModus()) {
-      size_t value=gameArea->GetFocusedCellMarks();
+    if (gameModel->IsCellInMarkModus()) {
+      size_t value=gameModel->GetFocusedCellMarks();
       size_t marks=0;
 
       for (size_t i=0; i<Sudoku::dimension; i++) {
@@ -361,7 +466,7 @@ public:
 
       if (marks==1) {
         mark->Enable();
-        mark->Set(gameArea->IsCellInMarkModus());
+        mark->Set(gameModel->IsCellInMarkModus());
       }
       else {
         mark->Set(marks>1);
@@ -376,7 +481,7 @@ public:
       }
     }
     else {
-      size_t value=gameArea->GetFocusedCellValue();
+      size_t value=gameModel->GetFocusedCellValue();
 
       for (size_t i=0; i<Sudoku::dimension; i++) {
         keypad[i]->Enable();
@@ -397,8 +502,9 @@ public:
 
   void UpdateCellFromKeypad()
   {
-    if (gameArea->GetFinishedModel()->Get() ||
-        !gameArea->IsFocusedCellEditable()) {
+    if (gameArea==NULL ||
+        gameModel->IsFinished() ||
+        !gameModel->IsFocusedCellEditable()) {
       // We ignore possible changes if there is no valid game
       return;
     }
@@ -419,20 +525,20 @@ public:
         }
       }
 
-      gameArea->SetFocusedCellMarks(value);
+      gameModel->SetFocusedCellMarks(value);
     }
     else {
       bool hasValue=false;
 
       for (size_t i=0; i<Sudoku::dimension; i++) {
         if (keypad[i].Valid() && keypad[i]->Get()) {
-          gameArea->SetFocusedCellValue(i+1);
+          gameModel->SetFocusedCellValue(i+1);
           hasValue=true;
         }
       }
 
       if (!hasValue) {
-        gameArea->SetFocusedCellValue(Sudoku::empty);
+        gameModel->SetFocusedCellValue(Sudoku::empty);
       }
     }
   }
@@ -451,27 +557,29 @@ public:
 
           riddle.CalculateWeight(solution,weight);
 
-          gameArea->SetGame(*game);
+          gameModel->Set(*game);
 
           delete game;
           game=NULL;
         }
       }
 
-      if (gameArea->GetFinishedModel()->Get()) {
+      if (gameModel->IsFinished()) {
         markEmptyAction->Disable();
         solveAction->Disable();
         statAction->Disable();
+        gameArea->GetHintMode()->Set(false);
+        gameArea->GetHintMode()->Disable();
       }
-
-      gameArea->GetFinishedModel()->On(false);
     }
     else if (model==GetClosedAction() &&
              GetClosedAction()->IsFinished()) {
-      if (!gameArea->GetFinishedModel()->Get()) {
+      if (!gameModel->IsFinished() &&
+        gameModel->IsValid()) {
         game=new Game();
-        gameArea->GetGame(*game);
+        gameModel->Get(*game);
       }
+
       SaveConfig();
     }
     else if (model==generateAction &&
@@ -486,16 +594,17 @@ public:
       markEmptyAction->Enable();
       solveAction->Enable();
       statAction->Enable();
-      gameArea->SetModel(riddle,solution);
-      gameArea->GetFinishedModel()->On(false);
+      gameArea->GetHintMode()->Set(false);
+      gameArea->GetHintMode()->Enable();
+
+      gameModel->Set(riddle,solution);
     }
     else if (model==generateTask.GetAbortedAction() &&
              generateTask.GetAbortedAction()->IsFinished()) {
-      gameArea->GetFinishedModel()->On(false);
     }
     else if (model==markEmptyAction &&
              markEmptyAction->IsFinished()) {
-      gameArea->MarkEmpty();
+      gameModel->MarkEmpty();
     }
     else if (model==solveAction &&
              solveAction->IsFinished() &&
@@ -507,32 +616,26 @@ public:
                                L"\n"
                                L"Solving the game cannot be undone!"),
                              _(L"SOLVE_BUTTONS",L"_Solve it!*|_Cancel^"))==0) {
-        gameArea->GetFinishedModel()->Off();
-        gameArea->Solve();
-        gameArea->GetFinishedModel()->On(false);
+        gameArea->GetHintMode()->Set(false);
+        gameArea->GetHintMode()->Disable();
         markEmptyAction->Disable();
         solveAction->Disable();
         statAction->Disable();
+
+        ignoreFinished=true;
+        gameModel->Solve();
+        ignoreFinished=false;
       }
     }
     else if (model==statAction &&
              statAction->IsFinished()) {
-      /*
-      if (GetWindow()->IsFullScreen()) {
-        GetWindow()->LeaveFullScreen();
-      }
-      else {
-        GetWindow()->SetFullScreen();
-      }
-      return;*/
-
       Sudoku game;
       Sudoku riddle;
       Sudoku solution;
 
-      gameArea->GetGame(game);
-      gameArea->GetRiddle(riddle);
-      gameArea->GetSolution(solution);
+      gameModel->Get(game);
+      gameModel->GetRiddle(riddle);
+      gameModel->GetSolution(solution);
 
       StatisticsDialog::Show(GetWindow(),game,riddle,solution);
     }
@@ -556,34 +659,41 @@ public:
              aboutAction->IsFinished()) {
       Lum::Dlg::About::Show(this,info);
     }
-    else if (model==gameArea->GetFinishedModel() &&
-             gameArea->GetFinishedModel()->Get()) {
-      Lum::Dlg::Msg::ShowOk(this,
-                            _(L"CONGRATS_TITLE",L"Yeeeaarrrr!!\n"),
-                            _(L"CONGRATS_BODY",
-                              L"It looks like you have successfully finished\n"
-                              L"the given puzzle!\n"
-                              L"Congratulations!"));
-      markEmptyAction->Disable();
-      solveAction->Disable();
-      statAction->Disable();
-      UpdateKeypad();
-    }
     else if (model==clearAction &&
              clearAction->IsFinished()) {
       if (!lock) {
         lock=true;
-        gameArea->SetFocusedCellValue(Sudoku::empty);
+        gameModel->SetFocusedCellValue(Sudoku::empty);
         UpdateKeypad();
         lock=false;
       }
     }
-    else if (model==gameArea->GetFocusChangedAction() &&
-             gameArea->GetFocusChangedAction()->IsFinished()) {
-      if (!lock) {
-        lock=true;
-        UpdateKeypad();
-        lock=false;
+    else if (model==gameModel) {
+      if (dynamic_cast<const GameModel::ValueChanged*>(&msg)!=NULL) {
+        if (!lock) {
+          lock=true;
+          UpdateKeypad();
+          lock=false;
+        }
+      }
+      else if (dynamic_cast<const GameModel::StateChanged*>(&msg)!=NULL) {
+        if (gameModel->IsFinished()) {
+          if (!ignoreFinished) {
+            Lum::Dlg::Msg::ShowOk(this,
+                                  _(L"CONGRATS_TITLE",L"Yeeeaarrrr!!\n"),
+                                  _(L"CONGRATS_BODY",
+                                    L"It looks like you have successfully finished\n"
+                                    L"the given puzzle!\n"
+                                    L"Congratulations!"));
+          }
+
+          gameArea->GetHintMode()->Set(false);
+          gameArea->GetHintMode()->Disable();
+          markEmptyAction->Disable();
+          solveAction->Disable();
+          statAction->Disable();
+          UpdateKeypad();
+        }
       }
     }
     else if (model==mark) {
@@ -616,6 +726,10 @@ class Main : public Lum::GUIApplication<MainDialog>
 public:
   bool Initialize()
   {
+#if defined(APP_DATADIR)
+    Lum::Manager::FileSystem::Instance()->SetApplicationDataDir(Lum::Base::StringToWString(APP_DATADIR));
+#endif
+
     info.SetProgram(Lum::Base::StringToWString(PACKAGE_NAME));
     info.SetVersion(Lum::Base::StringToWString(PACKAGE_VERSION));
     info.SetDescription(_(L"ABOUT_DESC",L"A simple Sudoku implementation..."));
@@ -624,10 +738,6 @@ public:
     info.SetCopyright(L"(c) 2006, Tim Teulings");
     info.SetLicense(L"GNU Public License");
 
-#if defined(APP_DATADIR)
-    Lum::Manager::FileSystem::Instance()->SetApplicationDataDir(Lum::Base::StringToWString(APP_DATADIR));
-#endif
-
     std::wstring appDataDir;
 
     if (Lum::Manager::FileSystem::Instance()->GetEntry(Lum::Manager::FileSystem::appDataDir,
